@@ -1,4 +1,5 @@
 import sys
+from collections import MutableMapping
 
 from django.db.models.fields import FieldDoesNotExist
 from django.db.models.options import get_verbose_name
@@ -20,73 +21,81 @@ class PkWrapper(object):
             setattr(self.obj, attr, value)
         super(PkWrapper, self).__setattr__(attr, value)
 
-class DocumentMetaWrapper(object):
+class DocumentMetaWrapper(MutableMapping):
     """
     Used to store mongoengine's _meta dict to make the document admin
     as compatible as possible to django's meta class on models. 
     """
-    index_background = None
-    collection = None
-    queryset_class = None 
-    allow_inheritance = None
-    max_size = None
-    ordering = None
-    id_field = None
-    indexes = None
-    index_drop_dups = None
-    unique_indexes = None
-    app_label = None
-    max_documents = None
+    _pk = None
+    pk_name = None
+    _app_label = None
     module_name = None
-    index_opts = None
-    verbose_name = None
-    verbose_name_plural = None
+    _verbose_name = None
     has_auto_field = False
+    object_name = None
     proxy = []
     parents = {}
     many_to_many = []
+    _field_cache = None
+    document = None
+    _meta = None
     
     def __init__(self, document):
         self.document = document
-        self.meta = document._meta
-        
-        self.init_from_meta()
-        
-        self.pk_name = self.id_field
-        
-        self.init_pk()
-        
-    def init_from_meta(self):
-        for attr, value in self.document._meta.iteritems():
-            if hasattr(self, attr):
-                setattr(self, attr, value)
+        self._meta = getattr(document, '_meta', {})
         
         try:
             self.object_name = self.document.__name__
         except AttributeError:
             self.object_name = self.document.__class__.__name__
-            
+        
         self.module_name = self.object_name.lower()
         
-        model_module = sys.modules[self.document.__module__]
-        self.app_label = model_module.__name__.split('.')[-2]
+        # EmbeddedDocuments don't have an id field.
+        try:
+            self.pk_name = self._meta['id_field']
+            self._init_pk()
+        except KeyError:
+            pass
+    
+    @property
+    def app_label(self):
+        if self._app_label is None:
+            model_module = sys.modules[self.document.__module__]
+            self._app_label = model_module.__name__.split('.')[-2]
+        return self._app_label
+            
+    @property
+    def verbose_name(self):
+        """
+        Returns the verbose name of the document.
         
-        if self.verbose_name is None:
-            self.verbose_name = capfirst(get_verbose_name(self.object_name))
-        
-        self.verbose_name_raw = self.verbose_name
-        
-        if self.verbose_name_plural is None:
-            self.verbose_name_plural = "%ss" % self.verbose_name
+        Checks the original meta dict first. If it is not found 
+        then generates a verbose name from from the object name.
+        """
+        if self._verbose_name is None:
+            try:
+                self._verbose_name = capfirst(get_verbose_name(self._meta['verbose_name']))
+            except KeyError:
+                self._verbose_name = capfirst(get_verbose_name(self.object_name))
+                
+        return self._verbose_name
+    
+    @property
+    def verbose_name_raw(self):
+        return self.verbose_name
+    
+    @property
+    def verbose_name_plural(self):
+        return "%ss" % self.verbose_name
     
     @property    
     def pk(self):
         if not hasattr(self._pk, 'attname'):
-            self.init_pk()
+            self._init_pk()
         return self._pk
-            
-            
-    def init_pk(self):
+        
+    def _init_pk(self):
         """
         Adds a wrapper around the documents pk field. The wrapper object gets the attributes
         django expects on the pk field, like name and attname.
@@ -105,10 +114,11 @@ class DocumentMetaWrapper(object):
                 
             self.document._pk_val = getattr(self.document, self.pk_name)
             # avoid circular import
-            from mongodbforms.util import patch_document
-            def _get_pk_val(self):
+            #from mongodbforms.util import patch_document
+            def get_pk_val():
                 return self._pk_val
-            patch_document(_get_pk_val, self.document)
+            #patch_document(_get_pk_val, self.document)
+            self.document._get_pk_val = get_pk_val
         except AttributeError:
             return      
                 
@@ -139,7 +149,7 @@ class DocumentMetaWrapper(object):
         try:
             try:
                 return self._field_cache[name]
-            except AttributeError:
+            except TypeError:
                 self._init_field_cache()
                 return self._field_cache[name]
         except KeyError:
@@ -148,14 +158,15 @@ class DocumentMetaWrapper(object):
             
         
     def _init_field_cache(self):
-        if not hasattr(self, '_field_cache'):
+        if self._field_cache is None:
             self._field_cache = {}
         
-        for f in self.document._fields.itervalues():
+        for f in self.document._fields.values():
             if isinstance(f, ReferenceField):
-                model = f.document_type
-                model._admin_opts = DocumentMetaWrapper(model)
-                self._field_cache[model._admin_opts.module_name] = (f, model, False, False)
+                document = f.document_type
+                document._meta = DocumentMetaWrapper(document)
+                document._admin_opts = document._meta
+                self._field_cache[document._meta.module_name] = (f, document, False, False)
             else:
                 self._field_cache[f.name] = (f, None, True, False)
                 
@@ -166,10 +177,37 @@ class DocumentMetaWrapper(object):
         Returns the requested field by name. Raises FieldDoesNotExist on error.
         """
         return self.get_field_by_name(name)[0]
-
-    def __getitem__(self, key):
-        return self.meta[key]
     
+    def __getattr__(self, name):
+        try:
+            return self._meta[name]
+        except KeyError:
+            raise AttributeError
+        
+    def __setattr__(self, name, value):
+        if not hasattr(self, name):
+            self._meta[name] = value
+        else:
+            super(DocumentMetaWrapper, self).__setattr__(name, value)
+    
+    def __contains__(self,key):
+        return key in self._meta
+    
+    def __getitem__(self, key):
+        return self._meta[key]
+    
+    def __setitem__(self, key, value):
+        self._meta[key] = value
+
+    def __delitem__(self, key):
+        return self._meta.__delitem__(key)
+
+    def __iter__(self):
+        return self._meta.__iter__()
+
+    def __len__(self):
+        return self._meta.__len__()
+
     def get(self, key, default=None):
         try:
             return self.__getitem__(key)
@@ -179,8 +217,8 @@ class DocumentMetaWrapper(object):
     def get_parent_list(self):
         return []
     
-    def get_all_related_objects(self, local_only=False, include_hidden=False):
+    def get_all_related_objects(self, *args, **kwargs):
         return []
 
     def iteritems(self):
-        return self.meta.iteritems()
+        return iter(self._meta.items())
