@@ -13,12 +13,18 @@ from django.forms.formsets import BaseFormSet, formset_factory
 from django.utils.translation import ugettext_lazy as _, ugettext
 from django.utils.text import capfirst
 
-from mongoengine.fields import ObjectIdField, ListField, ReferenceField
-from mongoengine.base import ValidationError
+from mongoengine.fields import ObjectIdField, ListField, ReferenceField, FileField, ImageField
+import collections
+try:
+    from mongoengine.base import ValidationError
+except ImportError:
+    from mongoengine.errors import ValidationError
 from mongoengine.connection import _get_db
 
-from fieldgenerator import MongoDefaultFormFieldGenerator
-from documentoptions import DocumentMetaWrapper
+from .fieldgenerator import MongoDefaultFormFieldGenerator
+from .documentoptions import DocumentMetaWrapper
+
+from .six import with_metaclass
 
 
 def _get_unique_filename(name):
@@ -27,7 +33,7 @@ def _get_unique_filename(name):
     count = itertools.count(1)
     while fs.exists(filename=name):
         # file_ext includes the dot.
-        name = os.path.join("%s_%s%s" % (file_root, count.next(), file_ext))
+        name = os.path.join("%s_%s%s" % (file_root, next(count), file_ext))
     return name
 
 def construct_instance(form, instance, fields=None, exclude=None, ignore=None):
@@ -36,7 +42,6 @@ def construct_instance(form, instance, fields=None, exclude=None, ignore=None):
     ``cleaned_data``, but does not save the returned instance to the
     database.
     """
-    from mongoengine.fields import FileField
     cleaned_data = form.cleaned_data
     file_field_list = []
     
@@ -44,7 +49,7 @@ def construct_instance(form, instance, fields=None, exclude=None, ignore=None):
     if isinstance(instance, type):
         instance = instance()
         
-    for f in instance._fields.itervalues():
+    for f in instance._fields.values():
         if isinstance(f, ObjectIdField):
             continue
         if not f.name in cleaned_data:
@@ -55,7 +60,7 @@ def construct_instance(form, instance, fields=None, exclude=None, ignore=None):
             continue
         # Defer saving file-type fields until after the other fields, so a
         # callable upload_to can use the values from other fields.
-        if isinstance(f, FileField):
+        if isinstance(f, FileField) or isinstance(f, ImageField):
             file_field_list.append(f)
         else:
             setattr(instance, f.name, cleaned_data[f.name])
@@ -99,7 +104,7 @@ def save_instance(form, instance, fields=None, fail_message='saved',
         # see BaseDocumentForm._post_clean for an explanation
         if hasattr(form, '_delete_before_save'):
             fields = instance._fields
-            new_fields = dict([(n, f) for n, f in fields.iteritems() if not n in form._delete_before_save])
+            new_fields = dict([(n, f) for n, f in fields.items() if not n in form._delete_before_save])
             if hasattr(instance, '_changed_fields'):
                 for field in form._delete_before_save:
                     instance._changed_fields.remove(field)
@@ -124,7 +129,7 @@ def document_to_dict(instance, fields=None, exclude=None):
     the ``fields`` argument.
     """
     data = {}
-    for f in instance._fields.itervalues():
+    for f in instance._fields.values():
         if fields and not f.name in fields:
             continue
         if exclude and f.name in exclude:
@@ -154,12 +159,12 @@ def fields_for_document(document, fields=None, exclude=None, widgets=None, \
     # they were defined on he document (at least with cPython) and I can't see 
     # any other way for now. Oh, yeah, it works because we sort on the memory address
     # and hope that the earlier fields have a lower address.
-    sorted_fields = sorted(document._fields.values(), key=lambda field: field.__hash__())
+    sorted_fields = sorted(list(document._fields.values()), key=lambda field: field.__hash__())
     
     for f in sorted_fields:
         if isinstance(f, ObjectIdField):
             continue
-        if isinstance(f, ListField) and not isinstance(f.field, ReferenceField):
+        if isinstance(f, ListField) and not (hasattr(f.field,'choices') or isinstance(f.field, ReferenceField)):
             continue
         if fields is not None and not f.name in fields:
             continue
@@ -170,10 +175,11 @@ def fields_for_document(document, fields=None, exclude=None, widgets=None, \
         else:
             kwargs = {}
 
-        formfield = field_generator.generate(f)
-        if formfield_callback is not None and not callable(formfield_callback):
+        if formfield_callback is None:
+            formfield = field_generator.generate(f, **kwargs)
+        elif not isinstance(formfield_callback, collections.Callable):
             raise TypeError('formfield_callback must be a function or callable')
-        elif formfield_callback is not None:
+        else:
             formfield = formfield_callback(f, **kwargs)
 
         if formfield:
@@ -195,10 +201,11 @@ class ModelFormOptions(object):
     def __init__(self, options=None):
         self.document = getattr(options, 'document', None)
         self.model = self.document
+        meta = getattr(self.document, '_meta', {})
         # set up the document meta wrapper if document meta is a dict
-        if self.document is not None and isinstance(self.document._meta, dict):
-            self.document._admin_opts = DocumentMetaWrapper(self.document)
-            self.document._meta = self.document._admin_opts
+        if self.document is not None and isinstance(meta, dict):
+            self.document._meta = DocumentMetaWrapper(self.document)
+            self.document._admin_opts = self.document._meta
         self.fields = getattr(options, 'fields', None)
         self.exclude = getattr(options, 'exclude', None)
         self.widgets = getattr(options, 'widgets', None)
@@ -230,7 +237,7 @@ class DocumentFormMetaclass(type):
             fields = fields_for_document(opts.document, opts.fields,
                             opts.exclude, opts.widgets, formfield_callback, formfield_generator)
             # make sure opts.fields doesn't specify an invalid field
-            none_document_fields = [k for k, v in fields.iteritems() if not v]
+            none_document_fields = [k for k, v in fields.items() if not v]
             missing_fields = set(none_document_fields) - \
                              set(declared_fields.keys())
             if missing_fields:
@@ -278,7 +285,7 @@ class BaseDocumentForm(BaseForm):
                                             error_class, label_suffix, empty_permitted)
 
     def _update_errors(self, message_dict):
-        for k, v in message_dict.items():
+        for k, v in list(message_dict.items()):
             if k != NON_FIELD_ERRORS:
                 self._errors.setdefault(k, self.error_class()).extend(v)
                 # Remove the data from the cleaned_data dict since it was invalid
@@ -297,7 +304,7 @@ class BaseDocumentForm(BaseForm):
         exclude = []
         # Build up a list of fields that should be excluded from model field
         # validation and unique checks.
-        for f in self.instance._fields.itervalues():
+        for f in self.instance._fields.values():
             field = f.name
             # Exclude fields that aren't on the form. The developer may be
             # adding these values to the model after form validation.
@@ -314,7 +321,7 @@ class BaseDocumentForm(BaseForm):
 
             # Exclude fields that failed form validation. There's no need for
             # the model fields to validate them as well.
-            elif field in self._errors.keys():
+            elif field in list(self._errors.keys()):
                 exclude.append(f.name)
 
             # Exclude empty fields that are not required by the form, if the
@@ -343,7 +350,7 @@ class BaseDocumentForm(BaseForm):
         # Clean the model instance's fields.
         to_delete = []
         try:
-            for f in self.instance._fields.itervalues():
+            for f in self.instance._fields.values():
                 value = getattr(self.instance, f.name)
                 if f.name not in exclude:
                     f.validate(value)
@@ -352,7 +359,7 @@ class BaseDocumentForm(BaseForm):
                     # that are not required. Clean them up here, though
                     # this is maybe not the right place :-)
                     to_delete.append(f.name)
-        except ValidationError, e:
+        except ValidationError as e:
             err = {f.name: [e.message]}
             self._update_errors(err)
         
@@ -368,7 +375,7 @@ class BaseDocumentForm(BaseForm):
         if hasattr(self.instance, 'clean'):
             try:
                 self.instance.clean()
-            except ValidationError, e:
+            except ValidationError as e:
                 self._update_errors({NON_FIELD_ERRORS: e.messages})
 
         # Validate uniqueness if needed.
@@ -382,7 +389,7 @@ class BaseDocumentForm(BaseForm):
         """
         errors = []
         exclude = self._get_validation_exclusions()
-        for f in self.instance._fields.itervalues():
+        for f in self.instance._fields.values():
             if f.unique and f.name not in exclude:
                 filter_kwargs = {
                     f.name: getattr(self.instance, f.name)
@@ -393,9 +400,9 @@ class BaseDocumentForm(BaseForm):
                 if self.instance.pk is not None:
                     qs = qs.filter(pk__ne=self.instance.pk)
                 if len(qs) > 0:
-                    message = _(u"%(model_name)s with this %(field_label)s already exists.") %  {
-                                'model_name': unicode(capfirst(self.instance._meta.verbose_name)),
-                                'field_label': unicode(pretty_name(f.name))
+                    message = _("%(model_name)s with this %(field_label)s already exists.") %  {
+                                'model_name': str(capfirst(self.instance._meta.verbose_name)),
+                                'field_label': str(pretty_name(f.name))
                     }
                     err_dict = {f.name: [message]}
                     self._update_errors(err_dict)
@@ -426,8 +433,8 @@ class BaseDocumentForm(BaseForm):
         return obj
     save.alters_data = True
 
-class DocumentForm(BaseDocumentForm):
-    __metaclass__ = DocumentFormMetaclass
+class DocumentForm(with_metaclass(DocumentFormMetaclass, BaseDocumentForm)):
+    pass
     
 def documentform_factory(document, form=DocumentForm, fields=None, exclude=None,
                        formfield_callback=None):
@@ -461,24 +468,30 @@ def documentform_factory(document, form=DocumentForm, fields=None, exclude=None,
     return DocumentFormMetaclass(class_name, (form,), form_class_attrs)
 
 
-class EmbeddedDocumentForm(BaseDocumentForm):
-    __metaclass__ = DocumentFormMetaclass
-    
+class EmbeddedDocumentForm(with_metaclass(DocumentFormMetaclass, BaseDocumentForm)):
     def __init__(self, parent_document, *args, **kwargs):
         super(EmbeddedDocumentForm, self).__init__(*args, **kwargs)
         self.parent_document = parent_document
-        if self._meta.embedded_field is not None and \
-                not hasattr(self.parent_document, self._meta.embedded_field):
+        if self._meta.embedded_field is not None and not \
+                self._meta.embedded_field in self.parent_document._fields:
             raise FieldError("Parent document must have field %s" % self._meta.embedded_field)
         
     def save(self, commit=True):
+        """If commit is True the embedded document is added to the parent
+        document. Otherwise the parent_document is left untouched and the
+        embedded is returned as usual.
+        """
         if self.errors:
             raise ValueError("The %s could not be saved because the data didn't"
                          " validate." % self.instance.__class__.__name__)
         
-        
         if commit:
-            l = getattr(self.parent_document, self._meta.embedded_field)
+            field = self.parent_document._fields.get(self._meta.embedded_field) 
+            if isinstance(field, ListField) and field.default is None:
+                default = []
+            else:
+                default = field.default
+            l = getattr(self.parent_document, self._meta.embedded_field, default)
             l.append(self.instance)
             setattr(self.parent_document, self._meta.embedded_field, l)
             self.parent_document.save() 
@@ -494,7 +507,7 @@ class BaseDocumentFormSet(BaseFormSet):
     def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
                  queryset=None, **kwargs):
         self.queryset = queryset
-        self._queryset = queryset
+        self._queryset = self.queryset
         self.initial = self.construct_initial()
         defaults = {'data': data, 'files': files, 'auto_id': auto_id, 
                     'prefix': prefix, 'initial': self.initial}
@@ -561,7 +574,7 @@ class BaseDocumentFormSet(BaseFormSet):
             "which must be unique for the %(lookup)s in %(date_field)s.") % {
             'field_name': date_check[2],
             'date_field': date_check[3],
-            'lookup': unicode(date_check[1]),
+            'lookup': str(date_check[1]),
         }
 
     def get_form_error(self):
@@ -682,4 +695,4 @@ def embeddedformset_factory(document, parent_document, form=EmbeddedDocumentForm
     }
     FormSet = inlineformset_factory(document, **kwargs)
     FormSet.parent_document = parent_document
-    return FormSet
\ No newline at end of file
+    return FormSet
