@@ -7,12 +7,16 @@ Wilson Júnior (wilsonpjunior@gmail.com).
 
 from django import forms
 from django.core.validators import EMPTY_VALUES
-from django.utils.translation import ugettext as _
-from pymongo.errors import InvalidId
-from pymongo.objectid import ObjectId
 from django.utils.encoding import smart_unicode, force_unicode
 from django.utils.translation import ugettext_lazy as _
 
+try:  # objectid was moved into bson in pymongo 1.9
+    from bson.objectid import ObjectId
+    from bson.errors import InvalidId
+except ImportError:
+    from pymongo.objectid import ObjectId
+    from pymongo.errors import InvalidId
+
 class MongoChoiceIterator(object):
     def __init__(self, field):
         self.field = field
@@ -20,7 +24,7 @@ class MongoChoiceIterator(object):
 
     def __iter__(self):
         if self.field.empty_label is not None:
-            yield (u"", self.field.empty_label)
+            yield ("", self.field.empty_label)
 
         for obj in self.queryset.all():
             yield self.choice(obj)
@@ -41,7 +45,7 @@ class ReferenceField(forms.ChoiceField):
     """
     Reference field for mongo forms. Inspired by `django.forms.models.ModelChoiceField`.
     """
-    def __init__(self, queryset, empty_label=u"---------",
+    def __init__(self, queryset, empty_label="---------",
                  *aargs, **kwaargs):
 
         forms.Field.__init__(self, *aargs, **kwaargs)
@@ -50,6 +54,12 @@ class ReferenceField(forms.ChoiceField):
 
     def _get_queryset(self):
         return self._queryset
+    
+    def _set_queryset(self, queryset):
+        self._queryset = queryset
+        self.widget.choices = self.choices
+
+    queryset = property(_get_queryset, _set_queryset)
 
     def prepare_value(self, value):
         if hasattr(value, '_meta'):
@@ -57,12 +67,6 @@ class ReferenceField(forms.ChoiceField):
 
         return super(ReferenceField, self).prepare_value(value)
 
-    def _set_queryset(self, queryset):
-        self._queryset = queryset
-        self.widget.choices = self.choices
-
-    queryset = property(_get_queryset, _set_queryset)
-
     def _get_choices(self):
         return MongoChoiceIterator(self)
 
@@ -77,8 +81,14 @@ class ReferenceField(forms.ChoiceField):
         return smart_unicode(obj)
 
     def clean(self, value):
-        if value in EMPTY_VALUES and not self.required:
-            return None
+        # Check for empty values. 
+        if value in EMPTY_VALUES:
+            # Raise exception if it's empty and required.
+            if self.required:
+                raise forms.ValidationError(self.error_messages['required'])
+            # If it's not required just ignore it.
+            else:
+                return None
 
         try:
             oid = ObjectId(value)
@@ -89,16 +99,26 @@ class ReferenceField(forms.ChoiceField):
         except (TypeError, InvalidId, self.queryset._document.DoesNotExist):
             raise forms.ValidationError(self.error_messages['invalid_choice'] % {'value':value})
         return obj
+    
+    # Fix for Django 1.4
+    # TODO: Test with older django versions
+    # from django-mongotools by wpjunior
+    # https://github.com/wpjunior/django-mongotools/
+    def __deepcopy__(self, memo):
+        result = super(forms.ChoiceField, self).__deepcopy__(memo)
+        result.queryset = result.queryset
+        result.empty_label = result.empty_label
+        return result
 
 class DocumentMultipleChoiceField(ReferenceField):
     """A MultipleChoiceField whose choices are a model QuerySet."""
     widget = forms.SelectMultiple
     hidden_widget = forms.MultipleHiddenInput
     default_error_messages = {
-        'list': _(u'Enter a list of values.'),
-        'invalid_choice': _(u'Select a valid choice. %s is not one of the'
-                            u' available choices.'),
-        'invalid_pk_value': _(u'"%s" is not a valid value for a primary key.')
+        'list': _('Enter a list of values.'),
+        'invalid_choice': _('Select a valid choice. %s is not one of the'
+                            ' available choices.'),
+        'invalid_pk_value': _('"%s" is not a valid value for a primary key.')
     }
 
     def __init__(self, queryset, *args, **kwargs):
